<?php
/*
 * Federation log viewer. Backend class
 */

class Backend {

	private $xml_parser;
	private $remotestatus;
	private $remotelines;
	private $currentline;

	function readLog($id, $lines) {
		global $sources, $defaultlines;

		if (!isset($sources[$id])) {
			return array("Log not found");
		} elseif ($sources[$id]['class'] == 'local') {
			return $this->readLocalLog($id, $lines);
		} else {
			return $this->readRemoteLog($id, $lines);
		}
	}

	/*
	 * Reads a local log file
	 */
	function readLocalLog($id, $lines) {
		global $sources, $tailPath;

        $ret = array();

		$fullpath = $sources[$id]['path'];

		$command = sprintf("%s -%d %s 2>&1", 
			$tailPath, $lines, $fullpath);

		$fp = popen($command, 'r');

		if (!$fp) {
			return array('Error creating pipe');
		} else {
			$result = '';
			while (!feof($fp)) {
				$line = fgets($fp, 4096);
                if ($line == FALSE) {
                    break;
                }
				$line = preg_replace('/[\r\n]+/', '', $line);
				$ret[] = $line;
			}
			pclose($fp);
		}
		return $ret;
	}

	/*
	 * Reads a remote log file
	 */
	function readRemoteLog($id, $lines) {
		global $sources, $timeout;

        $ret = array();

		// Read from http://..../log/$remoteId
		$base_url = $sources[$id]['path'] . '/log/'
			.  $sources[$id]['remoteId'];

		$url = $base_url . '?lines='.$lines;

		// cURL stuff
		$c = curl_init($url);

		if ($c === FALSE) {
			return array("Couldn't open remote log:", 
					curl_error($c));
		}

		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($c, CURLOPT_CONNECTTIMEOUT, $timeout);
		$xml = curl_exec($c);

		if ($xml === FALSE) {
			return array('Error opening '.$base_url);
		}
		curl_close($c);

		// XML parser
		$this->xml_parser = xml_parser_create();
		xml_parser_set_option($this->xml_parser, XML_OPTION_CASE_FOLDING, true);
		xml_set_element_handler($this->xml_parser, 
				array(&$this, "startElement"), 
				array(&$this, "endElement"));
		xml_set_character_data_handler($this->xml_parser, 
				array(&$this, "characterData"));


		// Preparation
		$this->remotelines = array();
		$this->remotestatus = array();

		if (!xml_parse($this->xml_parser, $xml)) {
			die(sprintf("XML error: %s at line %d",
					xml_error_string(xml_get_error_code($this->xml_parser)),
					xml_get_current_line_number($this->xml_parser)));
		}

		xml_parser_free($this->xml_parser);

		if (isset($this->remotestatus['error'])) {
			return array("Remote error: ".$this->remotestatus['error']);
		} else {
			return $this->remotelines;
		}
	}

	function startElement($parser, $name, $attrs) {
		// <status code="ok/error" message="xxxx" />
		if ($name == 'STATUS') {
			$this->remotestatus[$attrs['CODE']] = $attrs['MESSAGE'];
		} elseif ($name == 'LINE') {
			$this->currentline = "";
		}
	}

	function characterData($parser, $data) {
		$this->currentline .= $data;
	}

	function endElement($parser, $name) {
		if ($name == 'LINE') {
			// End of data
			$this->remotelines[] = $this->currentline;
		}
	}

}
?>
